from zemfrog.decorators import auto_status_code, authenticate
from zemfrog.helper import db_add, db_delete, db_update
from zemfrog.models import DefaultResponseSchema
from flask_apispec import marshal_with, use_kwargs
from {{ "" if main_app else ".." }}extensions.marshmallow import ma
from {{ "" if main_app else ".." }}{{src_model}} import {{name}}

class Create{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}

class Read{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}

class Update{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}

    __update__ = ma.Nested(Read{{name}}Schema(strict=True))

class Delete{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}

@authenticate
@marshal_with(Read{{name}}Schema(many=True), 200)
def get():
    """
    Read all data.
    """

    data = {{name}}.query.all()
    return data

@authenticate
@use_kwargs(Create{{name}}Schema(strict=True))
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 403)
@auto_status_code
def add(**json):
    """
    Add data.
    """

    found = {{name}}.query.filter_by(**json).first()
    if not found:
        model = {{name}}(**json)
        db_add(model)
        status_code = 200
        reason = "Successfully added data."

    else:
        status_code = 403
        reason = "Data already exists."

    return {
        "status_code": status_code,
        "reason": reason
    }

@authenticate
@use_kwargs(Update{{name}}Schema(strict=True))
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 404)
@marshal_with(DefaultResponseSchema, 403)
@auto_status_code
def update(**json):
    """
    Update data.
    """

    new_data = json.pop("__update__", None)
    if new_data and isinstance(new_data, dict):
        model = {{name}}.query.filter_by(**json).first()
        if not model:
            status_code = 404
            reason = "Data not found."

        else:
            db_update(model, **new_data)
            status_code = 200
            reason = "Successfully updating data."

    else:
        status_code = 403
        reason = "New data not found."

    return {
        "status_code": status_code,
        "reason": reason
    }

@authenticate
@use_kwargs(Delete{{name}}Schema(strict=True))
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 404)
@auto_status_code
def delete(**json):
    """
    Delete data.
    """

    model = {{name}}.query.filter_by(**json).first()
    if model:
        db_delete(model)
        status_code = 200
        reason = "Data deleted successfully."

    else:
        status_code = 404
        reason = "Data not found."

    return {
        "status_code": status_code,
        "reason": reason
    }


docs = {"tags": ["{{name}}"]}
endpoint = "{{url_prefix}}"
url_prefix = "/{{url_prefix}}"
routes = [
    ("/get", get, ["GET"]),
    ("/add", add, ["POST"]),
    ("/update", update, ["PUT"]),
    ("/delete", delete, ["DELETE"])
]



import re
from marshmallow import Schema, fields, validates, ValidationError


class DefaultResponseSchema(Schema):
    reason = fields.String()
    status_code = fields.Integer()


class LoginSuccessSchema(Schema):
    access_token = fields.String()


class LoginSchema(Schema):
    username = fields.Email(required=True)
    password = fields.String(required=True)


class RegisterSchema(LoginSchema):
    first_name = fields.String(required=True)
    last_name = fields.String(required=True)

    @validates("first_name")
    def validate_first_name(self, value):
        if not re.search(r"^([a-zA-Z]*)$", value):
            raise ValidationError("First name must be a character [a-zA-Z]")

    @validates("last_name")
    def validate_last_name(self, value):
        if not re.search(r"^([a-zA-Z]*)$", value):
            raise ValidationError("Last name must be a character [a-zA-Z]")


class RequestPasswordResetSchema(Schema):
    username = fields.Email(required=True)


class PasswordResetSchema(Schema):
    password = fields.String(required=True)


from {{ "" if main_app else ".." }}extensions.celery import celery
from {{ "" if main_app else ".." }}extensions.mail import mail


@celery.task
def send_email(*args, **kwds):
    mail.send_message(*args, **kwds)



import click
from flask.cli import with_appcontext
from ..generator import g_middleware


@click.group("middleware")
def group():
    """
    Middleware generator.
    """


@group.command()
@with_appcontext
@click.argument("name")
def new(name):
    """
    Create middleware.
    """

    g_middleware(name)


command = group



from {{ "" if main_app else ".." }}extensions.sqlalchemy import db
from sqlalchemy import Column, String, Integer, ForeignKey, DateTime, Boolean
from sqlalchemy.orm import relationship


class User(db.Model):
    """
    Simple user object.
    """

    id = Column(Integer, primary_key=True)
    first_name = Column(String, nullable=False)
    last_name = Column(String, nullable=False)
    name = Column(String, nullable=False)
    email = Column(String, nullable=False, unique=True)
    password = Column(String, nullable=False)
    register_at = Column(DateTime)
    confirmed = Column(Boolean)
    confirmed_at = Column(DateTime)
    logs = relationship("Log")
    roles = relationship("Role")


class Role(db.Model):
    """
    Role user.
    """

    id = Column(Integer, primary_key=True)
    user_id = Column(ForeignKey("user.id"))
    name = Column(String, nullable=False)


class Log(db.Model):
    """
    User activity log.
    """

    id = Column(Integer, primary_key=True)
    user_id = Column(ForeignKey("user.id"))
    login_at = Column(DateTime)
    logout_at = Column(DateTime)
    request_password_reset_at = Column(DateTime)
    updated_at = Column(DateTime)



"""
Reference: https://github.com/tiangolo/full-stack-flask-couchdb/tree/master/%7B%7Bcookiecutter.project_slug%7D%7D/backend/app/app/api/api_v1
"""

from apispec import APISpec
from apispec.ext.marshmallow import MarshmallowPlugin
from flask.app import Flask
from flask_apispec import FlaskApiSpec

docs = FlaskApiSpec(document_options=False)


def init_app(app: Flask):
    app.config.update(
        {
            "APISPEC_SPEC": APISpec(
                title=app.config.get("APISPEC_TITLE", "API Docs"),
                version=app.config.get("APISPEC_VERSION", "v1"),
                openapi_version=app.config.get("APISPEC_OAS_VERSION", "2.0"),
                plugins=[MarshmallowPlugin()],
                securityDefinitions=app.config.get("APISPEC_SECURITY_DEFINITIONS", {}),
            ),
        }
    )
    docs.init_app(app)


import click
from flask.cli import with_appcontext
from ..generator import g_error_handler


@click.group("handler")
def group():
    """
    Error handler generator.
    """


@group.command()
@with_appcontext
@click.argument("name")
def new(name):
    """
    Create error handler.
    """

    g_error_handler(name)


command = group



import os
from flask.app import Flask
from flask_migrate import Migrate
from .sqlalchemy import db

migrate = Migrate(db=db)


def init_app(app: Flask):
    directory = os.path.join(app.root_path, "migrations")
    migrate.init_app(app, directory=directory)



import os
from glob import glob
from importlib import import_module
from typing import Dict, List


import pkg_resources
import click
from flask import Flask
from flask.blueprints import Blueprint
from flask.cli import load_dotenv, routes_command, run_command, shell_command
from flask_apispec import FlaskApiSpec
from werkzeug.middleware.dispatcher import DispatcherMiddleware

from .exception import ZemfrogEnvironment
from .decorators import api_doc
from .helper import get_import_name, get_models, import_attr
from .repl import build_repl


def load_config(app: Flask):
    """
    Loads the configuration for your zemfrog application based on the environment
    ``ZEMFROG_ENV``, change your application environment in the file ``.flaskenv``.
    """

    path = os.path.join(app.root_path, ".flaskenv")
    load_dotenv(path)
    env = os.getenv("ZEMFROG_ENV")
    if not env:
        raise ZemfrogEnvironment("environment not found")

    import_name = get_import_name(app)
    app.config.from_object(import_name + "config." + env.capitalize())


def load_extensions(app: Flask):
    """
    The function to load all your flask extensions based on the ``EXTENSIONS`` configuration in config.py.
    """

    extensions = app.config.get("EXTENSIONS", [])
    import_name = get_import_name(app)
    for ext in extensions:
        ext = import_module(import_name + ext)
        init_func = getattr(ext, "init_app")
        init_func(app)


def load_models(app: Flask):
    """
    A function to load all your ORM models in the ``models`` directory.
    """

    app.models = {}
    true = app.config.get("CREATE_DB")
    import_name = get_import_name(app)
    if import_name:
        import_name = import_name.replace(".", "/")

    if true:
        models = [
            x.rsplit(".", 1)[0].replace(os.sep, ".")
            for x in glob(import_name + "models/**/*.py", recursive=True)
        ]
        for m in models:
            if "__init__" in m:
                m = m.replace(".__init__", "")
            mod = import_module(m)
            app.models[m] = get_models(mod)

        app.extensions["sqlalchemy"].db.create_all()


def load_commands(app: Flask):
    """
    A function to load all your commands and register them in the ``flask`` command.
    """

    commands = app.config.get("COMMANDS", [])
    import_name = get_import_name(app)
    for name in commands:
        try:
            n = import_name + name + ".command"
            cmd = import_attr(n)
        except ImportError:
            n = name + ".command"
            cmd = import_attr(n)

        app.cli.add_command(cmd)

    if import_name:
        for cmd in (run_command, shell_command, routes_command):
            app.cli.add_command(cmd)

        for ep in pkg_resources.iter_entry_points("flask.commands"):
            app.cli.add_command(ep.load(), ep.name)


def load_urls(app: Flask):
    """
    This function will load all urls in the main application.
    """

    import_name = get_import_name(app)
    routes = import_attr(import_name + "urls.routes")
    for url, view, methods in routes:
        app.add_url_rule(url, view_func=view, methods=methods)


def load_blueprints(app: Flask):
    """
    The function to load all blueprints based on the ``BLUEPRINTS`` configuration in config.py
    """

    blueprints = app.config.get("BLUEPRINTS", [])
    import_name = get_import_name(app)
    for name in blueprints:
        bp = import_name + name + ".routes.blueprint"
        bp: Blueprint = import_attr(bp)
        routes = import_name + name + ".urls.routes"
        routes = import_attr(routes)
        for url, view, methods in routes:
            bp.add_url_rule(url, view_func=view, methods=methods)

        app.register_blueprint(bp)


def load_middlewares(app: Flask):
    """
    Function to load all middlewares.
    """

    middlewares = app.config.get("MIDDLEWARES", [])
    import_name = get_import_name(app)
    for name in middlewares:
        name = import_name + name + ".init_middleware"
        middleware = import_attr(name)
        app.wsgi_app = middleware(app.wsgi_app)


def load_apis(app: Flask):
    """
    A function to load all of your API resources to flask based on the ``APIS`` configuration in config.py.
    """

    apis = app.config.get("APIS", [])
    import_name = get_import_name(app)
    api: Blueprint = import_attr(import_name + "api.api")
    for res in apis:
        res = import_module(import_name + res)
        endpoint = res.endpoint
        url_prefix = res.url_prefix
        routes = res.routes
        for detail in routes:
            route, view, methods = detail
            url = url_prefix + route
            e = endpoint + "_" + view.__name__
            api.add_url_rule(url, e, view_func=view, methods=methods)

    app.register_blueprint(api)


def load_error_handlers(app: Flask):
    """
    Function to load all error handlers
    """

    import_name = get_import_name(app)
    handlers = app.config.get("ERROR_HANDLERS", {})
    for code, view in handlers.items():
        view = import_attr(import_name + view + ".handler")
        app.register_error_handler(code, view)


def load_tasks(app: Flask):
    """
    Function to load all celery tasks based on ``TASKS`` configuration in config.py.
    """

    tasks = app.config.get("TASKS", [])
    import_name = get_import_name(app)
    for sv in tasks:
        import_module(import_name + sv)


def load_docs(app: Flask):
    """
    Function for creating api docs using ``flask-apispec``.
    """

    if not app.config.get("API_DOCS", False):
        return

    import_name = get_import_name(app)
    docs: FlaskApiSpec = import_attr(import_name + "extensions.apispec.docs")
    urls = import_module(import_name + "urls")
    docs_params = urls.docs
    routes = urls.routes
    for _, view, _ in routes:
        if docs_params:
            view = api_doc(**docs_params)(view)
        docs.register(view)

    apis = app.config.get("APIS", [])
    for res in apis:
        res = import_module(import_name + res)
        docs_params = res.docs
        routes = res.routes
        endpoint = res.endpoint
        for detail in routes:
            _, view, _ = detail
            e = endpoint + "_" + view.__name__
            if docs_params:
                view = api_doc(**docs_params)(view)
            docs.register(view, endpoint=e, blueprint="api")

    blueprints = app.config.get("BLUEPRINTS", [])
    for name in blueprints:
        bp = import_name + name + ".routes.blueprint"
        bp: Blueprint = import_attr(bp)
        urls = import_name + name + ".urls"
        urls = import_module(urls)
        docs_params = urls.docs
        routes = urls.routes
        for _, view, _ in routes:
            if docs_params:
                view = api_doc(**docs_params)(view)
            docs.register(view, blueprint=name)


def load_apps(app: Flask):
    """
    Load all applications and combine them together using ``DispatcherMiddleware``.
    """

    apps: List[Dict] = app.config.get("APPS", [])
    mounts = {}
    for a in apps:
        if not isinstance(a, dict):
            a = {"name": a}

        name = a["name"]
        path = a.get("path", "/" + name)
        help = a.get("help", "")
        yourapp: Flask = import_attr(name + ".wsgi.app")

        @click.command(name)
        def cli():
            build_repl(yourapp)

        cli.help = help
        app.cli.add_command(cli, name)
        mounts[path] = yourapp

    app.wsgi_app = DispatcherMiddleware(app.wsgi_app, mounts)



from functools import wraps
from typing import Callable
from flask import current_app
from flask_apispec import doc
from flask_jwt_extended import jwt_required


def auto_status_code(func: Callable) -> Callable:
    """
    Decorator to generate HTTP status response code automatically
    according to the key ``status_code`` in json.

    :param func: Your view function.

    """

    @wraps(func)
    def wrapper(*args, **kwds):
        result = func(*args, **kwds)
        status_code = 200
        if isinstance(result, dict):
            status_code = result.get("status_code", 200)

        if not isinstance(result, (tuple, list)):
            result = result, status_code

        return result

    return wrapper


def api_doc(**kwds) -> Callable:
    """
    Decorator for adding API documentation through the documentation in the view function.
    """

    def wrapper(func: Callable):
        d = kwds.pop("description", func.__doc__ or "")
        kwds["description"] = d
        func = doc(**kwds)(func)
        return func

    return wrapper


def authenticate(func: Callable) -> Callable:
    """
    Decorator to add jwt view authentication.
    Reference: https://github.com/tiangolo/full-stack-flask-couchdb/tree/master/%7B%7Bcookiecutter.project_slug%7D%7D/backend/app/app/api/api_v1.
    """

    func = api_doc(security=current_app.config.get("APISPEC_SECURITY_PARAMS", []))(
        jwt_required(func)
    )
    return func



import click

from flask.cli import with_appcontext
from ..generator import g_blueprint


@click.group("blueprint")
def group():
    """
    Blueprint generator.
    """


@group.command()
@with_appcontext
@click.argument("name")
def new(name):
    """
    Create blueprint.
    """

    g_blueprint(name)


command = group



from flask import Flask
from . import loader
from celery import Celery


def make_celery(app: Flask) -> Celery:
    """
    Creating a celery application for flask applications.
    Source: https://flask.palletsprojects.com/en/1.1.x/patterns/celery/#configure
    """

    celery = Celery(
        app.import_name,
        backend=app.config["CELERY_RESULT_BACKEND"],
        broker=app.config["CELERY_BROKER_URL"],
    )
    celery.conf.update(app.config)

    class ContextTask(celery.Task):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return self.run(*args, **kwargs)

    celery.Task = ContextTask
    return celery


def create_app(name: str) -> Flask:
    """
    Functions to build your flask application and load all configurations.

    :param name: import name.

    """

    app = Flask(name)
    with app.app_context():
        loader.load_config(app)
        loader.load_extensions(app)
        loader.load_models(app)
        loader.load_urls(app)
        loader.load_blueprints(app)
        loader.load_middlewares(app)
        loader.load_apis(app)
        loader.load_error_handlers(app)
        loader.load_commands(app)
        loader.load_tasks(app)
        loader.load_docs(app)
        loader.load_apps(app)

    return app



import click
from flask.cli import with_appcontext
from ..generator import g_command


@click.group("command")
def group():
    """
    Command generator.
    """


@group.command()
@with_appcontext
@click.argument("name")
def new(name):
    """
    Create command.
    """

    g_command(name)


command = group



from flask import jsonify
from flask_jwt_extended import JWTManager
from {{ "" if main_app else ".." }}models.user import User

jwt = JWTManager()


@jwt.user_loader_callback_loader
def user_loader_callback(identity):
    user = User.query.filter_by(email=identity).first()
    return user


@jwt.user_loader_error_loader
def custom_user_loader_error(identity):
    ret = {"reason": "User {} not found".format(identity), "status_code": 404}
    return jsonify(ret), 404


init_app = jwt.init_app
